/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.kernel.scripting;

/**
 * Implementation of the script api from the job scheduler.
 *
 * This class is a general implementation of the scheduler script api for using with
 * all script languages supported by the javax interface for scripting.
 * 
 * see http://java.sun.com/developer/technicalArticles/J2SE/Desktop/scripting/
 * see https://scripting.dev.java.net/
 *
 * @author Stefan Schaedlich
 * @version 1.0 - 2010-12-17
 * <div class="sos_branding">
 *   <p>(c) 2010 SOS GmbH - Berlin (<a style='color:silver' href='http://www.sos-berlin.com'>http://www.sos-berlin.com</a>)</p>
 * </div>
 */

// import org.apache.log4j.Appender;
import org.apache.log4j.Logger;
// import com.sos.JSHelper.Logging.JobSchedulerLog4JAppender;

public final class APIModuleInstance extends ScriptInstance {

    private final Logger logger = Logger.getLogger(APIModuleInstance.class);
	private static final String LANGUAGE_PREFIX = "javax.script:";
	private static final String SCHEDULER_PROCESS = "spooler_process";

	public APIModuleInstance(String scriptLanguage, String sourcecode) {
		super(getScriptLanguage(scriptLanguage),sourcecode);
	}

	/**
	 * The script language to use
     * @return String
	 */
	private static String getScriptLanguage(String scriptlanguage) {
		return scriptlanguage.toLowerCase().replace(LANGUAGE_PREFIX, "");
	}

	@Override
	public Object call(String functionName) {
		return callFunction(functionName, new Object[] {});
	}

	public boolean callBoolean(String functionname, boolean defaultResult) {
		boolean result = defaultResult;
        try {
            result = callBoolean(functionname, new Object[] {defaultResult});
        } catch (NoSuchMethodException e) {
            result = defaultResult;
        }
        return result;
    }

    /**
     * Call a script function
     * It's just the same like the call method of the superclass, but the error handling is different. The JS
     * calls the script for any function of the api (scheduler_init, scheduler_open etc.), but it is not necessary
     * that the functions are present in the script.
     *
     * scheduler_process has to be present. If not the whole script will run without functions.
     *
     * @return Object - with the result of the script
     *
     * @param functionName
     * @param params
     */
    private Object callFunction(String functionName, Object[] params) {
        Object result = null;
        try {
            logger.debug("call for function " + functionName + " (" + functionName + ")");
            result = super.call(functionName, params);
            logger.debug("result is " + result);
        } catch (NoSuchMethodException e) {
            logger.debug("function " + functionName + " not found");
            if (functionName.equals(SCHEDULER_PROCESS)) {
                super.call();
                result = Boolean.FALSE;
            }
        }
        return result;
    }

}
