/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.util.xml;

import com.google.common.collect.AbstractIterator;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import javax.annotation.Nullable;

import static org.w3c.dom.Node.ELEMENT_NODE;

public class NamedSiblingElementIterator extends AbstractIterator<Element> {
    private final String name;
    @Nullable private Element next;

    public NamedSiblingElementIterator(String elementName, Node firstSibling) {
        name = elementName;
        next = sameOrNextElement(firstSibling);
    }

    @Override protected Element computeNext() {
        Element result = next;
        if (next != null)
            next = sameOrNextElement(next.getNextSibling());
        return result == null? endOfData() : result;
    }

    @Nullable protected Element sameOrNextElement(@Nullable Node node) {
        Node n = node;
        while (n != null && (n.getNodeType() != ELEMENT_NODE || !n.getNodeName().equals(name)))
            n = n.getNextSibling();
        return (Element)n;
    }
}
