/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package sos.scheduler.editor.app;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

public class Messages {
    private static final String   BUNDLE_NAME = "sos.scheduler.editor.messages"; //$NON-NLS-1$

    private static ResourceBundle RESOURCE_BUNDLE;


    // private static ResourceBundle RESOURCE_BUNDLE = ResourceBundle
    // .getBundle(BUNDLE_NAME, Locale.GERMAN);

    private Messages() {
    }


    public static boolean setResource(Locale locale) {
        try {
            RESOURCE_BUNDLE = ResourceBundle.getBundle(BUNDLE_NAME, locale);
        } catch (MissingResourceException e) {
            return false;
        }
        return true;
    }

 
    public static String getBundle() {
        return BUNDLE_NAME;
    }


    public static String getString(String key) {
        try {
            return RESOURCE_BUNDLE.getString(key);
        } catch (MissingResourceException e) {
            return '!' + key + '!';
        }
    }


    public static String getString(String key, String variable) {
        return getString(key, new String[] { variable });
    }


    public static String getString(String key, Object[] values) {
        try {
            return MessageFormat.format(RESOURCE_BUNDLE.getString(key), values);
        } catch (MissingResourceException e) {
            return '!' + key + '!';
        }
    }


    public static boolean hasMessage(String key) {
        try {
            String msg = RESOURCE_BUNDLE.getString(key);
            return msg != null && !msg.equals("");
        } catch (MissingResourceException e) {
            return false;
        }
    }


    public static String getTooltip(String key) {
        try {
            String msg = RESOURCE_BUNDLE.getString("tooltip." + key);
            return msg != null && !msg.equals("") ? msg : null;
        } catch (Exception e) {
            return null;
        }
    }
}
