/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
#include "spooler.h"

namespace sos {
namespace scheduler {
namespace folder {

//-----------------------------------------------------------------------Dependencies::Dependencies

Dependencies::Dependencies( File_based_subsystem* subsystem )
:
    _zero_(this+1),
    _subsystem( subsystem )
{
}

//----------------------------------------------------------------------Dependencies::~Dependencies
    
Dependencies::~Dependencies()
{
}

//----------------------------------------------------------------------Dependencies::add_requisite

void Dependencies::add_requisite( Dependant* dependant, const string& missings_path )
{
    Z_LOG2("JS-644", obj_name() << "::" << Z_FUNCTION << " " << dependant->obj_name() << " '" << missings_path << "'\n");
    _path_requestors_map[ _subsystem->normalized_path( missings_path ) ].insert( dependant );
}

//-------------------------------------------------------------------Dependencies::remove_requisite

void Dependencies::remove_requisite( Dependant* dependant, const string& missings_path )
{
    Z_LOG2("JS-644", obj_name() << "::" << Z_FUNCTION << " '" << missings_path << "'\n");
    Requestor_set&  requestors_set = _path_requestors_map[ missings_path ];
    
    requestors_set.erase( dependant );
    if( requestors_set.empty() )  _path_requestors_map.erase( _subsystem->normalized_path( missings_path ) );
}

//----------------------------------------------------------Dependencies::announce_requisite_loaded

void Dependencies::announce_requisite_loaded( File_based* found_missing )
{
    Z_LOG2("JS-644", obj_name() << "::" << Z_FUNCTION << " " << found_missing->obj_name() << "\n");
    assert( found_missing->subsystem() == _subsystem );

    Path_requestors_map::iterator it = _path_requestors_map.find( found_missing->normalized_path() );

    if( it != _path_requestors_map.end() ) {
        Requestor_set& requestor_set = it->second;

        Z_FOR_EACH( Requestor_set, requestor_set, it2 ) {
            Requestor_set::iterator next_it2  = it2;  next_it2++;
            Dependant*              dependant = *it2;
        
            try {
                Z_LOG2("JS-644", Z_FUNCTION << " " << dependant->obj_name() << "->on_requisite_loaded()\n");
                dependant->on_requisite_loaded( found_missing );
            }
            catch( exception& x ) {
                string m = message_string("SCHEDULER-459", found_missing->obj_name(), x.what());
                Z_LOG2("JS-644", "ERROR " << Z_FUNCTION << " " << m << "\n");
                dependant->log()->error(m);
                found_missing->log()->error(m);
            }
        }
    }
}

//---------------------------------------------------Dependencies::announce_requisite_to_be_removed

bool Dependencies::announce_requisite_to_be_removed( File_based* to_be_removed )
{
    assert( to_be_removed->subsystem() == _subsystem );
    Z_LOG2("JS-644", obj_name() << "::" << Z_FUNCTION << " " << to_be_removed->obj_name() << "\n");

    bool result = true;

    Path_requestors_map::iterator it = _path_requestors_map.find( to_be_removed->normalized_path() );

    if( it != _path_requestors_map.end() ) {
        Requestor_set& requestor_set = it->second;

        Z_FOR_EACH( Requestor_set, requestor_set, it2 ) {
            Requestor_set::iterator next_it2  = it2;  next_it2++;
            Dependant*              dependant = *it2;
        
            Z_DEBUG_ONLY(Z_LOG2("JS-644", Z_FUNCTION << " " << dependant->obj_name() << "->on_requisite_to_be_removed( " << to_be_removed->obj_name() << " )\n" ));
            result = dependant->on_requisite_to_be_removed( to_be_removed );
            if( !result )  break;
        }
    }

    return result;
}

//---------------------------------------------------------Dependencies::announce_requisite_removed

void Dependencies::announce_requisite_removed( File_based* to_be_removed )
{
    assert( to_be_removed->subsystem() == _subsystem );

    Path_requestors_map::iterator it = _path_requestors_map.find( to_be_removed->normalized_path() );

    if( it != _path_requestors_map.end() )
    {
        Requestor_set& requestor_set = it->second;

        Z_FOR_EACH( Requestor_set, requestor_set, it2 )
        {
            Requestor_set::iterator next_it2  = it2;  next_it2++;
            Dependant*              dependant = *it2;
        
            Z_DEBUG_ONLY(Z_LOG2("JS-644", obj_name() << "::" << Z_FUNCTION << " " << dependant->obj_name() << " on_requisite_to_be_removed( " << to_be_removed->obj_name() << " )\n"));
            dependant->on_requisite_removed( to_be_removed );
        }
    }
}

//---------------------------------------------------------------------------Dependencies::obj_name

string Dependencies::obj_name() const {
    return S() << "Dependencies(" << _subsystem->obj_name() << ")";
}

}}} //namespace sos::scheduler::folder
