/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
#include "spooler.h"
#include "log_cache_Request.h"
#include "log_cache_Request_cache.h"

namespace sos {
namespace scheduler {
namespace log {
namespace cache {

struct Request_impl : Request {
    private: Fill_zero              _zero_;
    private: Request_cache* const   _cache;
    private: ptr<Prefix_log> const  _log;
    public: int64                   _number;

    public: Request_impl(Request_cache* cache, Prefix_log* log, int64 number) :
        _zero_(this+1),
        _cache(cache),
        _log(log),
        _number(number)
    {}

    public: ~Request_impl() {
        _log->close_file();
        _log->set_append(true);   // Damit nach erneutem ffnen die Datei fortgesetzt wird.
    }

    public: Prefix_log* log() const {  return _log; }
    public: int64 number() const { return _number; }
    public: string obj_name() const { return S() << "log::cache::Request(" << _log->obj_name() << ")"; }
};

ptr<Request> Request::of(Request_cache* cache, Prefix_log* log, int64 number) {
    log->try_reopen_file();
    ptr<Request_impl> result = log->file_is_opened()? Z_NEW(Request_impl(cache, log, number)) : NULL;
    return +result;
}

}}}}
