/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: threads.cxx 11394 2005-04-03 08:30:29Z jz $

#include "zschimmer.h"
#include "threads.h"

namespace zschimmer {

//-----------------------------------------------------------------------------Simple_event::signal

void Simple_event::signal( const string& name )
{
    Z_MUTEX( _mutex )
    {
        _signal_name = name;
        _signaled = true;
    }
}

//-----------------------------------------------------------------------Simple_event::async_signal

void Simple_event::async_signal( const char* )
{
    _signaled = true;
}

//----------------------------------------------------------------Simple_event::signaled_then_reset

bool Simple_event::signaled_then_reset()
{
    bool signaled = false;

    Z_MUTEX( _mutex )
    {
        signaled = _signaled;
        _signal_name = "";
        _signaled = false;
    }

    return signaled;
}

//-------------------------------------------------------------------------------Simple_event::reset

void Simple_event::reset()
{
    Z_MUTEX( _mutex )
    {
        _signal_name = "";
        _signaled = false;
    }
}

//--------------------------------------------------------------------------------Simple_event::wait

void Simple_event::wait()
{
    throw_xc( "Simple_event::wait" );
}

//--------------------------------------------------------------------------------Simple_event::wait

bool Simple_event::wait( double )
{
    throw_xc( "Simple_event::wait" );
}

//----------------------------------------------------------------------------Callback_event::signal

void Callback_event::signal(const string& name) {
    _signalable->on_event_signaled();
    Simple_event::signal(name);
}

//-------------------------------------------------------------------------------------------------

} //namespace zschimmer
