/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: z_posix_thread.h 13322 2008-01-27 15:27:19Z jz $

#ifndef ZSCHIMMER_Z_POSIX_THREAD_H
#define ZSCHIMMER_Z_POSIX_THREAD_H

#include "base.h"

#ifdef Z_UNIX

#include <pthread.h>
#include "threads_base.h"
#include "log.h"


namespace zschimmer {
namespace posix {

//-------------------------------------------------------------------------------------------Thread

struct Thread : zschimmer::Thread_base, Non_cloneable
{
  //typedef pthread_t           Id;


                                Thread                      ( const string& name = "" )             : Thread_base(name), _zero_(this+1) {}
                               ~Thread                      ();

    void                        thread_close                ();
    void                        thread_start                ();

  //bool                        thread_wait_for_termination ( double wait_time = INT_MAX );
    void                        thread_wait_for_termination ();

    bool                        thread_is_running           ()                                      { return _thread_is_running; }
    Id                          thread_id                   ()                                      { return _thread_id; }
  //void                    set_thread_id                   ( Id id )                               { _thread_id = id; }            // Nur sinnvoll, wenn das Objekt mal nicht als Thread luft (z.B. Task_subsystem)
    pthread_t                   thread_handle               ()                                      { return _pthread_handle; }

  //bool                        try_set_thread_priority     ( int pri )                             { return SetThreadPriority( _thread_handle._handle, pri ) != 0; }
  //int                         thread_priority             ()                                      { return GetThreadPriority( _thread_handle._handle ); }
  //virtual string              thread_as_text              () const;

  protected:
    friend void*                thread_function             ( void* );
    virtual int                 thread_main                 () = 0;
    void                        thread_init                 ();
    void                        thread_exit                 ( int exit_code );

  private:
    Fill_zero                  _zero_;
  //pthread_attr_t             _pthread_attr;
    pthread_t                  _pthread_handle;
    Id                         _thread_id;                  // Kopie von _pthread_handle oder von auen gesetzt (dann _pthread_handle == NULL, fr Spooler-Threads)
  //Event                      _thread_event;
    bool                       _thread_is_running;
    int                        _thread_pid;
};


inline Thread::Id               current_thread_id           ()                                      { return pthread_self(); }

//-----------------------------------------------------------------------------Thread_specific_data

struct Thread_specific_data      
{
                                Thread_specific_data        ()                                      : _key( (pthread_key_t)-1 ) {}
                               ~Thread_specific_data        ()                                      { free(); }

    void                        allocate                    ();
    void                        free                        ();
    void                        set                         ( void* value ) const                   { pthread_setspecific( _key, value ); }
    void*                       get                         () const                                { return pthread_getspecific( _key ); }

    bool                        is_allocated                () const                                { return _key != (pthread_key_t)-1; }

    pthread_key_t              _key;
};

//-------------------------------------------------------------------------------------------------

} //namespace posix
} //namespace zschimmer

#endif
#endif
